"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var utils_1 = require("./utils");
/**
 * @file File defining the SequentialEvent class
 * @licence GPLv3
 * @author Gerkin
 */
/**
 * Event emitter that guarantees sequential execution of handlers. Each handler may return a **Promise**.
 *
 * @see {@link https://nodejs.org/api/events.html Node EventEmitter}.
 */
var SequentialEvent = /** @class */ (function () {
    /**
     * Constructs a new SequentialEvent.
     *
     * @author Gerkin
     */
    function SequentialEvent() {
        this.__events = {};
    }
    /**
     * Triggers each corresponding handlers in sequence.
     *
     * @param event Name of the event to trigger
     * @param args Arguments to pass to each handler. Note that those arguments will be appended with previous handler resolution value, if any.
     * @author Gerkin
     */
    SequentialEvent.prototype.emit = function (event) {
        var args = [];
        for (var _i = 1; _i < arguments.length; _i++) {
            args[_i - 1] = arguments[_i];
        }
        var events = this.__events;
        var handler = events[event];
        if (!handler) {
            return Promise.resolve();
        }
        var retPromise = utils_1.emitHandlers(handler, this, args);
        return retPromise;
    };
    SequentialEvent.prototype.off = function (events, callback) {
        var _this = this;
        var _events = this.__events;
        if (events === true) {
            this.__events = {};
            return this;
        }
        else if (typeof events === 'string' && typeof callback === 'undefined') {
            events.split(' ').forEach(function (event) { return (_this.__events[event] = []); });
            return this;
        }
        var eventsHash = utils_1.castArgsToEventsHash(events, callback);
        utils_1.forEachObj(eventsHash, function (handler, event) {
            utils_1.removeEventListener(_events[event], handler);
        });
        return this;
    };
    SequentialEvent.prototype.on = function (events, callback) {
        var _events = this.__events;
        var eventsHash = utils_1.castArgsToEventsHash(events, callback);
        utils_1.forEachObj(eventsHash, function (handler, event) {
            utils_1.addEventListener(_events, event, handler);
        });
        return this;
    };
    SequentialEvent.prototype.once = function (events, callback) {
        var _this = this;
        var _events = this.__events;
        var eventsHash = utils_1.castArgsToEventsHash(events, callback);
        utils_1.forEachObj(eventsHash, function (handlers, event) {
            utils_1.addEventListener(_events, event, handlers.map(function (eventHandler) { return utils_1.onceify(_this, event, eventHandler); }));
        });
        return this;
    };
    /**
     * Check if this instance has listeners for the provided event. Alias for `hasEvent`
     *
     * @param event Name of the event to check
     * @author Gerkin
     */
    SequentialEvent.prototype.has = function (event) {
        return this.hasEvent(event);
    };
    /**
     * Check if this instance has listeners for the provided event
     *
     * @param event Name of the event to check
     * @author Gerkin
     */
    SequentialEvent.prototype.hasEvent = function (event) {
        return this.__events.hasOwnProperty(event) && this.__events[event].length > 0;
    };
    return SequentialEvent;
}());
exports.SequentialEvent = SequentialEvent;
//# sourceMappingURL=sequential-event.js.map